#include "gamelogic.h"
#include "accelerometer.h"
#include "scorecalculator.h"
#include "gamelogicconstants.h"
#include "bluetooth.h"
#include "devicevibrator.h"
#include <QtDeclarative>

GameLogic::GameLogic(QObject *parent) : QObject(parent) {
}

void GameLogic::initialize(QmlApplicationViewer* qav) {
    this->mainLoopTimer = new QTimer(this);
    this->viewer = qav;
    this->mainLoopTimer->start(17);

    this->waypoints.append(QString("POINT2"));
    this->waypoints.append(QString("HAMAKO"));
    this->accelerometerAccumulator=0;

    connect(this->mainLoopTimer, SIGNAL(timeout()), this, SLOT(mainLoop()));

    ScoreCalculator::instance()->start();

    this->penalty=0;
    this->state = "NORMAL";
    this->signalPercentage=0;
}

GameLogic::~GameLogic() {
    delete this->mainLoopTimer;
}

///
///Main loop for the game logic
///
void GameLogic::mainLoop() {
    qreal accelerometerData = Accelerometer::instance()->getAccelerometerAccelerationDelta();
    QMap<QString,qint16> deviceMap = Bluetooth::instance()->getDeviceMap();

    this->accelerometerAccumulator += accelerometerData;

    if(this->penalty == 0) {
        bool isOK = evaluateAccelerometerData();
        if(isOK) {
            bool moreWaypoints = evaluateDeviceMapData(deviceMap);
            if(moreWaypoints == false) {
                this->mainLoopTimer->stop();
                disconnect();
            }
        }
    } else {
        --this->penalty;
    }

    this->accelerometerAccumulator *= GameLogicConstants::ACCELEROMETER_ACCUMULATOR_DECREASERATE;
}
///
///Evaluates accelerometerdata, should the accelerometeracculumator be over STUN_TRESHOLD, the player should be stunned
///Returns false if player should be stunned, otherwise true
///
bool GameLogic::evaluateAccelerometerData() {
    if(this->accelerometerAccumulator > 100) this->accelerometerAccumulator = 100;
    bool isOK = true;
    if(this->accelerometerAccumulator > GameLogicConstants::STUN_THRESHOLD) {
        isOK=false;
        this->penalty = GameLogicConstants::STUN_FRAMES;
        this->state = "FAIL";
        DeviceVibrator::instance()->rumble(RumbleStrength::STRONG);
    } else {
        this->state = "NORMAL";
    }

    return isOK;
}

///
///Evaluates devicemapdata.
///Returns true if there are waypoints left or pingvin is close to a waypoint, false if all the waypoints have been done
///
bool GameLogic::evaluateDeviceMapData(QMap<QString, qint16> &deviceMap) {
    if(this->waypoints.count() == 0) return false;
    QMapIterator<QString,qint16> it(deviceMap);
    while(it.hasNext()) {
        it.next();
        if(it.key() == this->waypoints.front()) {

            qint16 sig = it.value();
            if(sig > -70) sig = -70;
            else if(sig < -100) sig = -100;
            sig += 100;
            this->signalPercentage = sig;
            this->signalPercentage = this->signalPercentage / 30;

            if(it.value() > GameLogicConstants::WAYPOINT_DONE_THRESHOLD) {
                DeviceVibrator::instance()->rumble();
                waypoints.pop_front();
                if(waypoints.count() == 0) {
                    this->state = "VICTORY";
                    return false;
                } else {
                    this->state = "WAYPOINT";
                    this->penalty = 100;
                    return true;
                }
            } else if(it.value() > GameLogicConstants::WAYPOINT_CLOSE_THRESHOLD) {
                this->state = "HAPPY";
                return true;
            }
        }
    }
    return true;
}

qreal GameLogic::getSkyOpacity() {
    qreal percentage = 1-this->accelerometerAccumulator / GameLogicConstants::STUN_THRESHOLD;
    if(percentage > 1) percentage = 1;
    return percentage;
}

QString GameLogic::getState() {
    return this->state;
}

QQueue<QString>& GameLogic::getWayPoints()
{
    return this->waypoints;
}

qreal GameLogic::getSignalPercentage() {
    //qDebug(QString::number(this->signalPercentage).toAscii());
    return this->signalPercentage;
}


